# from os.path import join

import typer

filename_option = typer.Argument(
    "/home/leynier/projects/viera-academy-app/ProjectSettings/ProjectSettings.asset",
    # join(".", "ProjectSettings", "ProjectSettings.asset"),
    exists=True,
    file_okay=True,
    dir_okay=False,
    writable=True,
    readable=True,
    resolve_path=True,
    help="Path of ProjectSettings.asset file of the Unity project",
    envvar=["UVM_FILENAME"],
)



from pathlib import Path
from typing import Optional

import typer

from ..core import Version
from ..utils import filename_option

app = typer.Typer(help="Command to manage the version code")


@app.command(name="get", help="Get the version code")
def code_get(filename: Optional[Path] = filename_option):
    version = Version.get(str(filename))
    typer.echo(f"Code: {version.code}")
    typer.echo(f"Version: {version}")


@app.command(name="setup", help="Increase the version code to the next")
def code_set_up(filename: Optional[Path] = filename_option):
    version = Version.get(str(filename))
    version.set_code_up()
    version.set(str(filename))
    typer.echo(f"Version: {version}")


@app.command(name="set", help="Set the version code")
def code_set(
    number: int = typer.Argument(..., help="Version code"),
    filename: Optional[Path] = filename_option,
):
    version = Version.get(str(filename))
    version.set_code(number)
    version.set(str(filename))
    typer.echo(f"Version: {version}")



from pathlib import Path
from typing import Optional

import typer

from .commands.build_command import app as build_app
from .commands.code_command import app as code_app
from .commands.major_command import app as major_app
from .commands.minor_command import app as minor_app
from .commands.patch_command import app as patch_app
from .core import Version
from .utils import filename_option

app = typer.Typer(help="Unity Version Manager CLI implemented with Python and Typer")

app.add_typer(code_app, name="code")
app.add_typer(build_app, name="build")
app.add_typer(patch_app, name="patch")
app.add_typer(minor_app, name="minor")
app.add_typer(major_app, name="major")


@app.command(name="get", help="Get the version")
def get(filename: Optional[Path] = filename_option):
    version = Version.get(str(filename))
    typer.echo(f"Version: {version}")


@app.command(name="set", help="Set the version")
def set(
    arg: str = typer.Argument(
        ...,
        help='The format must be "*.*.*b*(*)" where * is the major, minor, patch, '
        + "build and code respectively and all integers.",
    ),
    filename: Optional[Path] = filename_option,
):
    version = Version.parse(arg)
    if version is None:
        typer.echo(
            "Version format is invalid.\n"
            + 'The format must be "*.*.*b*(*)" where * is the major,\n'
            + "minor, patch, build and code respectively and all integers.",
            err=True,
        )
    else:
        version.set(str(filename))
        typer.echo(f"Version: {version}")



from typing import List, Optional


class Version:
    def __init__(self, major: int, minor: int, patch: int, build: int, code: int):
        self.major = major
        self.minor = minor
        self.patch = patch
        self.build = build
        self.code = code

    def __str__(self) -> str:
        return f"{self.major}.{self.minor}.{self.patch}b{self.build}({self.code})"

    def set_major(self, number: int):
        self.major = number

    def set_minor(self, number: int):
        self.minor = number

    def set_patch(self, number: int):
        self.patch = number

    def set_build(self, number: int):
        self.build = number

    def set_code(self, number: int):
        self.code = number

    def set_major_up(self):
        self.set_major(self.major + 1)
        self.set_minor(0)
        self.set_patch(0)
        self.set_build(0)
        self.set_code_up()

    def set_minor_up(self):
        self.set_minor(self.minor + 1)
        self.set_patch(0)
        self.set_build(0)
        self.set_code_up()

    def set_patch_up(self):
        self.set_patch(self.patch + 1)
        self.set_build(0)
        self.set_code_up()

    def set_build_up(self):
        self.set_build(self.build + 1)
        self.set_code_up()

    def set_code_up(self):
        self.code += 1

    @staticmethod
    def get(filename: str):
        major = -1
        minor = -1
        patch = -1
        build = -1
        code = -1
        with open(filename, mode="r") as file:
            mark = False
            for line in file.readlines():
                if "bundleVersion: " in line:
                    content = line.split(": ")[1]
                    content_splited = content.split(".")
                    if len(content_splited) != 3:
                        raise Exception(
                            "Version should be x.y.z format and "
                            + "x, y and z should be an integers"
                        )
                    major_str, minor_str, patch_str = content_splited
                    try:
                        major = int(major_str)
                        minor = int(minor_str)
                        patch = int(patch_str)
                    except ValueError:
                        raise Exception(
                            "Version should be x.y.z format and "
                            + "x, y and z should be an integers"
                        )
                elif "iPhone: " in line and mark:
                    content = line.split(": ")[1]
                    try:
                        build = int(content)
                    except ValueError:
                        raise Exception("Build should be an integer")
                    mark = False
                elif "AndroidBundleVersionCode: " in line:
                    content = line.split(": ")[1]
                    try:
                        code = int(content)
                    except ValueError:
                        raise Exception("Code should be an integer")
                mark = mark or "buildNumber:" in line
                if (
                    major != -1
                    and minor != -1
                    and patch != -1
                    and build != -1
                    and code != -1
                ):
                    break
        version = Version(major, minor, patch, build, code)
        if major == -1 or minor == -1 or patch == -1 or build == -1 or code == -1:
            raise Exception(f"Error parsing file: {version}")
        return version

    def set(self, filename: str):
        lines: List[str] = []
        with open(filename, mode="r") as file:
            mark = False
            for line in file.readlines():
                if "bundleVersion: " in line:
                    content = line.split(":")[0]
                    lines.append(f"{content}: {self.major}.{self.minor}.{self.patch}\n")
                elif "iPhone: " in line and mark:
                    content = line.split(":")[0]
                    lines.append(f"{content}: {self.build}\n")
                    mark = False
                elif "AndroidBundleVersionCode: " in line:
                    content = line.split(":")[0]
                    lines.append(f"{content}: {self.code}\n")
                else:
                    lines.append(line)
                mark = mark or "buildNumber:" in line
        with open(filename, mode="w") as file:
            file.writelines(lines)

    @staticmethod
    def parse(version: str) -> Optional["Version"]:
        if not version.endswith(")"):
            return None
        try:
            major = int(version.split(".")[0])
            minor = int(version.split(".")[1])
            patch = int(version.split(".")[2].split("b")[0])
            build = int(version.split("b")[1].split("(")[0])
            code = int(version.split("(")[1].strip(")"))
            return Version(major, minor, patch, build, code)
        except Exception:
            return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['uvm', 'uvm.commands']

package_data = \
{'': ['*']}

install_requires = \
['typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['uvm = uvm.main:app']}

setup_kwargs = {
    'name': 'uvm',
    'version': '1.0.2',
    'description': 'Unity Version Manager CLI implemented with Python and Typer',
    'long_description': '# Unity Version Manager\n\n[![License: MIT](https://img.shields.io/badge/License-MIT-green.svg)](https://opensource.org/licenses/MIT)\n[![Test](https://github.com/educup/uvm/workflows/CI/badge.svg)](https://github.com/educup/uvm/actions?query=workflow%3ACI)\n[![codecov](https://codecov.io/gh/educup/uvm/branch/main/graph/badge.svg?token=Z1MEEL3EAB)](https://codecov.io/gh/educup/uvm)\n[![Version](https://img.shields.io/pypi/v/uvm?color=%2334D058&label=Version)](https://pypi.org/project/uvm)\n[![Last commit](https://img.shields.io/github/last-commit/educup/uvm.svg?style=flat)](https://github.com/educup/uvm/commits)\n[![GitHub commit activity](https://img.shields.io/github/commit-activity/m/educup/uvm)](https://github.com/educup/uvm/commits)\n[![Github Stars](https://img.shields.io/github/stars/educup/uvm?style=flat&logo=github)](https://github.com/educup/uvm/stargazers)\n[![Github Forks](https://img.shields.io/github/forks/educup/uvm?style=flat&logo=github)](https://github.com/educup/uvm/network/members)\n[![Github Watchers](https://img.shields.io/github/watchers/educup/uvm?style=flat&logo=github)](https://github.com/educup/uvm)\n[![Website](https://img.shields.io/website?up_message=online&url=https%3A%2F%2Feducup.github.io/uvm)](https://educup.github.io/uvm)\n[![GitHub contributors](https://img.shields.io/github/contributors/educup/uvm)](https://github.com/educup/uvm/graphs/contributors)\n\n[Unity](https://unity.com) Version Manager CLI implemented with [Python](https://www.python.org) and [Typer](https://typer.tiangolo.com).\n\n**Usage**:\n\n```console\n$ uvm [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--install-completion`: Install completion for the current shell.\n* `--show-completion`: Show completion for the current shell, to copy it or customize the installation.\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `build`: Command to manage the version build\n* `code`: Command to manage the version code\n* `get`: Get the version\n* `major`: Command to manage the version major\n* `minor`: Command to manage the version minor\n* `patch`: Command to manage the version patch\n* `set`: Set the version\n\n## `uvm build`\n\nCommand to manage the version build\n\n**Usage**:\n\n```console\n$ uvm build [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `get`: Get the version build\n* `set`: Set the version build\n* `setup`: Increase the version build to the next\n\n### `uvm build get`\n\nGet the version build\n\n**Usage**:\n\n```console\n$ uvm build get [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm build set`\n\nSet the version build\n\n**Usage**:\n\n```console\n$ uvm build set [OPTIONS] NUMBER [FILENAME]\n```\n\n**Arguments**:\n\n* `NUMBER`: Version build  [required]\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm build setup`\n\nIncrease the version build to the next\n\n**Usage**:\n\n```console\n$ uvm build setup [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n## `uvm code`\n\nCommand to manage the version code\n\n**Usage**:\n\n```console\n$ uvm code [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `get`: Get the version code\n* `set`: Set the version code\n* `setup`: Increase the version code to the next\n\n### `uvm code get`\n\nGet the version code\n\n**Usage**:\n\n```console\n$ uvm code get [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm code set`\n\nSet the version code\n\n**Usage**:\n\n```console\n$ uvm code set [OPTIONS] NUMBER [FILENAME]\n```\n\n**Arguments**:\n\n* `NUMBER`: Version code  [required]\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm code setup`\n\nIncrease the version code to the next\n\n**Usage**:\n\n```console\n$ uvm code setup [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n## `uvm get`\n\nGet the version\n\n**Usage**:\n\n```console\n$ uvm get [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n## `uvm major`\n\nCommand to manage the version major\n\n**Usage**:\n\n```console\n$ uvm major [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `get`: Get the version major\n* `set`: Set the version major\n* `setup`: Increase the version major to the next\n\n### `uvm major get`\n\nGet the version major\n\n**Usage**:\n\n```console\n$ uvm major get [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm major set`\n\nSet the version major\n\n**Usage**:\n\n```console\n$ uvm major set [OPTIONS] NUMBER [FILENAME]\n```\n\n**Arguments**:\n\n* `NUMBER`: Version major  [required]\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm major setup`\n\nIncrease the version major to the next\n\n**Usage**:\n\n```console\n$ uvm major setup [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n## `uvm minor`\n\nCommand to manage the version minor\n\n**Usage**:\n\n```console\n$ uvm minor [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `get`: Get the version minor\n* `set`: Set the version minor\n* `setup`: Increase the version minor to the next\n\n### `uvm minor get`\n\nGet the version minor\n\n**Usage**:\n\n```console\n$ uvm minor get [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm minor set`\n\nSet the version minor\n\n**Usage**:\n\n```console\n$ uvm minor set [OPTIONS] NUMBER [FILENAME]\n```\n\n**Arguments**:\n\n* `NUMBER`: Version minor  [required]\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm minor setup`\n\nIncrease the version minor to the next\n\n**Usage**:\n\n```console\n$ uvm minor setup [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n## `uvm patch`\n\nCommand to manage the version patch\n\n**Usage**:\n\n```console\n$ uvm patch [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `get`: Get the version patch\n* `set`: Set the version patch\n* `setup`: Increase the version patch to the next\n\n### `uvm patch get`\n\nGet the version patch\n\n**Usage**:\n\n```console\n$ uvm patch get [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm patch set`\n\nSet the version patch\n\n**Usage**:\n\n```console\n$ uvm patch set [OPTIONS] NUMBER [FILENAME]\n```\n\n**Arguments**:\n\n* `NUMBER`: Version patch  [required]\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n### `uvm patch setup`\n\nIncrease the version patch to the next\n\n**Usage**:\n\n```console\n$ uvm patch setup [OPTIONS] [FILENAME]\n```\n\n**Arguments**:\n\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n\n## `uvm set`\n\nSet the version\n\n**Usage**:\n\n```console\n$ uvm set [OPTIONS] ARG [FILENAME]\n```\n\n**Arguments**:\n\n* `ARG`: The format must be `"*.*.*b*(*)"` where * is the major, minor, patch, build and code respectively and all integers.  [required]\n* `[FILENAME]`: Path of ProjectSettings.asset file of the Unity project  [envvar: UVM_FILENAME;default: ./ProjectSettings/ProjectSettings.asset]\n\n**Options**:\n\n* `--help`: Show this message and exit.\n',
    'author': 'educup',
    'author_email': 'hello@educup.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/educup/uvm',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
